@@ -42,9 +42,7 @@ from cddm.print_tools import print1,print2, print_frame_rate, enable_prints, dis
 import time
 from functools import reduce 
 from cddm.fft import _fft
-from cddm.avg import weight_from_data, weighted_sum
-from cddm._core_nb import log_interpolate,\
-   _cross_corr_fft_regular, _cross_corr_fft, \
+from cddm._core_nb import _cross_corr_fft_regular, _cross_corr_fft, \
     _auto_corr_fft_regular,_auto_corr_fft,\
   _cross_corr_regular, _cross_corr, _cross_corr_vec, \
   _cross_diff_regular,_cross_diff,_cross_diff_vec, \
@@ -52,54 +50,11 @@ from cddm._core_nb import log_interpolate,\
   _auto_diff_regular,_auto_diff, _auto_diff_vec,\
   _cross_sum_regular,_cross_sum,_cross_sum_vec,\
   _cross_sum_fft,_fill_ones,_cross_sum_rfft, abs2,\
-  _add_count_cross,_add_count_auto,\
-  _normalize_cdiff_1,_normalize_cdiff_3,\
-  _normalize_ccorr_0,_normalize_ccorr_1,_normalize_ccorr_2,_normalize_ccorr_3
+  _add_count_cross,_add_count_auto
 
-NORM_BASELINE = 0
-"""baseline normalization flag"""
-
-NORM_COMPENSATED = 1
-"""compensated normalization (cross-diff) flag"""
-
-NORM_SUBTRACTED = 2
-"""background subtraction normalization flag"""
-
-NORM_WEIGHTED = 4
-"""weighted normalization flag"""
-
-def norm_flags(compensated = False, subtracted = False, weighted = False):
-    """Return normalization flags from the parameters.
-    
-    Parameters
-    ----------
-    compensated : bool
-        Whether to set COMPENSATED normalization flag
-    subtracted : bool
-        Whether to set SUBTRACTED normalization flag
-    weighted : bool
-        Whether to set WEIGHTED normalization flag
-    
-    Returns
-    -------
-    norm : int
-        Normalization flagse
-        
-    Examples
-    --------
-    >>> norm_flags(True, False, True)
-    5
-    >>> norm_flags(True, True, False)
-    3
-    """
-    norm = NORM_BASELINE
-    if compensated == True:
-        norm = norm | NORM_COMPENSATED
-    if subtracted == True:
-        norm = norm | NORM_SUBTRACTED
-    if weighted == True:
-        norm = norm | NORM_WEIGHTED
-    return norm
+#imported for backward compatibility
+from cddm.norm import NORM_STANDARD, NORM_SUBTRACTED, NORM_STRUCTURED, NORM_WEIGHTED, NORM_COMPENSATED
+from cddm.norm import weight_from_data, weighted_sum, scale_factor, normalize, norm_flags, take_data
 
 #--------------------------
 #Core correlation functions
@@ -1010,9 +965,9 @@ def _default_method(method, n):
 
 def _default_norm(norm, method, cross = True):
     if method == "diff":
-        supported = (1,3) if cross else (1,)
+        supported = (NORM_STRUCTURED,NORM_STRUCTURED|NORM_SUBTRACTED) if cross else (NORM_STRUCTURED,)
     else:
-        supported = (0,1,2,3,4,5,7,6) #so that 6 is default
+        supported = (1,2,3,5,6,7)
     if norm is None:
         norm = supported[-1]
     if norm not in supported:
@@ -1110,7 +1065,7 @@ def acorr(f, t = None, fs = None,  n = None,  norm = None,
     else:
         _sum = auto_sum
         
-    if (norm & NORM_COMPENSATED or norm & NORM_WEIGHTED) and correlate:
+    if (norm & NORM_STRUCTURED) and correlate:
         if fs is None:
             fs =  abs2(f)
         else:
@@ -1255,7 +1210,7 @@ def ccorr(f1,f2,t1 = None, t2 = None,  n = None,
         _sum = cross_sum
 
         
-    if (norm & NORM_COMPENSATED or norm & NORM_WEIGHTED) and correlate:
+    if (norm & NORM_STRUCTURED) and correlate:
         print2("... tau sum square")
         if f1s is None:
             f1s =  abs2(f1)
@@ -1457,410 +1412,7 @@ def iacorr(data, t = None, n = None, norm = 0, method = "corr", count = None,
     if stats == True:
         return data, bg, var
     else:
-        return data
-    
-
-def _norm_from_ccorr_data(data, norm = None):
-    """Determines normalization type from ccorr data"""
-    try:
-        d, c, sq, s1, s2 = data
-    except ValueError:
-        raise ValueError("Not a valid correlation data")
-        
-    if s1 is None:
-        if sq is None:
-            default_norm = 0
-            available_norm = 0
-        else:
-            default_norm = 4
-            available_norm = 5
-    else:
-        if sq is None:
-            default_norm = 2
-            available_norm = 2
-        else:
-            default_norm = 6
-            available_norm = 7
-    if norm is None:
-        return default_norm
-    else:
-        if norm not in range(8):
-            raise ValueError("Normalization mode must be in range(8)")
-        if  (norm & available_norm) == norm:
-            return norm
-        else:
-            raise ValueError("Normalization mode incompatible with input data.")
-
-def _norm_from_cdiff_data(data, norm = None):
-    """Determines normalization type from ccorr data"""
-    try:
-        d, c, s1, s2 = data
-    except:
-        raise ValueError("Not a valid difference data")
-    if (s1 is None or s2 is None):
-        default_norm = 1
-    else:
-        default_norm = 3 
-    if norm is None:
-        return default_norm
-    else:
-        if norm not in (1,3):
-            raise ValueError("Normalization mode can be 1 or 3")
-        if  (norm & default_norm) == norm:
-            return norm
-        else:
-            raise ValueError("Normalization mode incompatible with input data.")
-
-def _default_norm_from_data(data, method, norm = None):
-    if method == "corr":
-        norm = _norm_from_ccorr_data(data, norm)
-    else:
-        norm = _norm_from_cdiff_data(data, norm)
-    return norm
-    
-
-def _diff2corr(data, variance, mask):
-    offset = _variance2offset(variance, mask)        
-    data *= (-0.5)
-    data += offset
-    return data
-    
-def _corr2diff(data, variance, mask):
-    offset = _variance2offset(variance, mask) * 2
-    data *= (-2)
-    data += offset
-    return data
-
-def _variance2offset(variance, mask):
-    #offset for norm == 2 type of normalization
-    return scale_factor(variance, mask)[..., None]
-    
-def scale_factor(variance, mask = None):
-    """Computes the normalization scaling factor from the variance data.
-    
-    You can divide the computed correlation data with this factor to normalize
-    data between (0,1) for correlation mode, or (0,2) for difference mode.
-    
-    Parameters
-    ----------
-    variance : (ndarray, ndarray) or ndarray
-        A variance data (as returned from :func:`.stats`)
-    mask : ndarray
-        A boolean mask array, if computation was performed on masked data,
-        this applys mask to the variance data.
-        
-    Returns
-    -------
-    scale : ndarray
-        A scaling factor for normalization
-    """
-    if variance is None:
-        raise ValueError("You must provide variance data for normalization")
-    try:
-        v1, v2 = variance
-        scale = 0.5*np.asarray((v1 + v2))
-    except:
-        scale = np.asarray(variance)  
-    if mask is None:
-        return scale
-    else:
-        return scale[mask]
-
-def _inspect_background(background, norm, mask):
-    if background is not None:
-        try:
-            bg1, bg2 = background
-        except:
-            bg1, bg2 = background, background
-            
-        bg1, bg2 = np.asarray(bg1,CDTYPE),np.asarray(bg2,CDTYPE)
-        bg1 = bg1[...,None] #add dimension for broadcasting
-        bg2 = bg2[...,None]
-        
-        if mask is not None:
-            bg1 = bg1[mask,:]
-            bg2 = bg2[mask,:]
-    elif norm > 0:
-        raise ValueError("You must provide background data for normalization.")
-    else:
-        bg1, bg2 = 0j,0j
-    return bg1, bg2
-
-def take_data(data, mask):
-    """Selects correlation(difference) data at given masked indices.
-    
-    Parameters
-    ----------
-    data : tuple of ndarrays
-        Data tuple as returned by `ccorr` and `acorr` functions
-    mask : ndarray
-        A boolean frame mask array 
-    
-    Returns
-    -------
-    out : tuple
-        Same data structure as input data, but with all arrays in data masked
-        with the provided mask array.
-    """
-    def _mask(i,data):
-        if i !=1:
-            data = data[...,mask,:] if data is not None else None
-        return data
-        
-    return tuple((_mask(i,d) for (i,d) in enumerate(data)))
-
-def _method_from_data(data):
-    try:
-        if len(data) == 5:
-            return "corr" 
-        elif len(data) == 4:
-            return "diff"
-        else:
-            1/0
-    except:
-        raise ValueError("Not a valid correlation data")
-        
-def _inspect_mode(mode):
-    if mode in ("corr","diff"):
-        return mode
-    else:
-        raise ValueError("Invalid mode")
-    
-def _inspect_scale(scale):
-    try:
-        return bool(scale)
-    except:
-        raise ValueError("Invalid scale")
-        
-def normalize(data, background = None, variance = None, norm = None,  mode = "corr", 
-              scale = False, mask = None, out = None):
-    """Normalizes correlation (difference) data. Data must be data as returned
-    from ccorr or acorr functions. 
-    
-    Except forthe most basic normalization, background and variance data must be provided.
-    Tou can use :func:`stats` to compute background and variance data.
-    
-    Parameters
-    ----------
-    data : tuple of ndarrays
-        Input data, a length 4 (difference data) or length 5 tuple (correlation data)
-    background : (ndarray, ndarray) or ndarray, optional
-        Background (mean) of the frame(s) in k-space
-    variance : (ndarray, ndarray) or ndarray, optional
-        Variance of the frame(s) in k-space
-    norm : int, optional
-        Normalization type (0:baseline,1:compensation,2:bg subtract,
-        3: compensation + bg subtract). Input data must support the chosen
-        normalization, otherwise exception is raised. If not given it is chosen
-        based on the input data.
-    mode : str, optional
-        Representation mode: either "corr" (default) for correlation function,
-        or "diff" for image structure function (image difference).
-    scale : bool, optional
-        If specified, performs scaling so that data is scaled beteween 0 and 1.
-        This works in connection with variance, which must be provided.
-    mask : ndarray, optional
-        An array of bools indicating which k-values should we select. If not 
-        given, compute at every k-value.
-    out : ndarray, optional
-        Output array
-        
-    Returns
-    -------
-    out : ndarray
-        Normalized data.
-    """
-    print1("Normalizing...")
-    #determine what kind of data is there ('diff' or 'corr')
-    method = _method_from_data(data)
-    scale = _inspect_scale(scale)
-    mode = _inspect_mode(mode)
-    
-    #determine default normalization if not specified by user
-    norm = _default_norm_from_data(data, method, norm)
-
-#    if len(data[1].shape) > 1 and norm & NORM_WEIGHTED:
-#        #if multilevel data, subtracted or baseline is best, so force it here.
-#        norm = norm & NORM_SUBTRACTED
-
-    # scale factor for normalization 
-    if scale == True:
-        _scale_factor = scale_factor(variance,mask)
-    
-    print2("   * background : {}".format(background is not None))
-    print2("   * variance   : {}".format(variance is not None))
-    print2("   * norm       : {}".format(norm))
-    print2("   * scale      : {}".format(scale))
-    print2("   * mode       : {}".format(mode))
-    print2("   * mask       : {}".format(mask is not None))
-    
-    if (norm & NORM_WEIGHTED):
-        level = disable_prints()
-        norm_comp = (norm & NORM_SUBTRACTED)|NORM_COMPENSATED
-        comp_data = normalize(data, background, variance, norm = norm_comp,  mode = mode, 
-              scale = scale, mask = mask)
-        norm_base = norm & NORM_SUBTRACTED
-        base_data = normalize(data, background, variance, norm = norm_base,  mode = mode, 
-              scale = scale, mask = mask)
-        
-        _multilevel = True if len(data[1].shape) > 1 else False
-        
-        x_avg,y_avg = _data_estimator(comp_data, size = 8, n = 3, multilevel = _multilevel)
-        
-        _scale_factor = 1. if scale == True else scale_factor(variance,mask)
-        
-        if _multilevel:
-            shape = np.array(comp_data.shape)
-            shape[1:-1] = 1
-            _x_interp = np.arange(comp_data.shape[-1])
-            x_interp = np.empty(shape = shape, dtype = int)
-            for x_level in x_interp:
-                x_level[...] = _x_interp
-                _x_interp *= 2
-        else:
-            x_interp = np.arange(data[1].shape[-1])
-        
-        weight = weight_from_data(y_avg, scale_factor = _scale_factor, mode = mode)
-        weight = log_interpolate(x_interp,x_avg, weight)
-        enable_prints(level) 
-        return weighted_sum(base_data,comp_data, weight)
-        
-    count = data[1]
-    
-    if mask is not None:
-        data = take_data(data, mask)
-    
-    #dimensions of correlation data (the first element of the data tuple)
-    ndim = data[0].ndim
-            
-    bg1, bg2 = _inspect_background(background, norm, mask)
-    
-    #need to add dimensions to count for broadcasting
-    #for 2D data, this is equivalent to count = count[...,None,None,:]
-    ndiff = ndim - count.ndim
-    for i in range(ndiff):
-        count = np.expand_dims(count,-2)
-
-    if norm == NORM_BASELINE:
-        result = _normalize_ccorr_0(data[0], count, bg1, bg2, out = out)
-        if mode == "diff":
-            result = _corr2diff(result, variance, mask)
-    
-    elif norm == NORM_COMPENSATED:
-        if method == "corr":
-            offset = _variance2offset(variance, mask)
-            result = _normalize_ccorr_1(data[0], count, bg1, bg2, data[2],  out = out)
-            result += offset
-            if mode == "diff":
-                result = _corr2diff(result, variance, mask)
-        
-        else:
-            d = bg2 - bg1
-            result = _normalize_cdiff_1(data[0], count, d, out = out)
-            if mode == "corr":
-                result = _diff2corr(result, variance, mask)
-
-    elif norm == NORM_SUBTRACTED:
-        m1 = data[3]
-        m2 = data[4] if data[4] is not None else m1
-        result = _normalize_ccorr_2(data[0], count, bg1, bg2, m1,m2, out = out)
-        if mode == "diff":
-            result = _corr2diff(result, variance, mask)
-
-    elif norm == NORM_COMPENSATED|NORM_SUBTRACTED:
-        if method == "corr":
-            offset = _variance2offset(variance, mask)
-            m1 = data[3]
-            m2 = data[4] if data[4] is not None else m1
-            result = _normalize_ccorr_3(data[0], count, bg1, bg2, data[2], m1, m2,  out = out)
-            result += offset
-            if mode == "diff":
-                result = _corr2diff(result, variance, mask)
-        else:
-            d = bg2 - bg1
-            result = _normalize_cdiff_3(data[0], count, d, data[2], data[3],out = out)
-
-            if mode == "corr":
-                result = _diff2corr(result, variance, mask)
-    if scale == True:
-        result /= _scale_factor[...,None]
-    return result
-
-#def weight_from_data(base_data, comp_data,  scale_factor = 1., mode = "corr", norm = NORM_BASELINE, multitau = False):
-#    comp_data_avg = average(comp_data)
-#    base_data_avg = average(base_data)
-#
-#    comp_data_avg[...,0] =  scale_factor if mode == "corr" else 0.
-#    median(comp_data_avg,comp_data_avg)
-#    comp_data_avg[...,0] =  scale_factor if mode == "corr" else 0.
-#    median(comp_data_avg,comp_data_avg)
-#    comp_data_avg[...,0] =  scale_factor if mode == "corr" else 0.
-#            
-#    weight = base_weight(comp_data_avg, scale_factor = scale_factor, mode = mode)
-#    
-#    #mask =  weight > 0.5
-#    #comp_data_avg[mask] =  base_data_avg[mask]
-#    #median(comp_data_avg,comp_data_avg)
-#    
-#    #weight = base_weight(comp_data_avg, scale_factor = scale_factor, mode = mode)
-#
-#    if norm & NORM_COMPENSATED:
-#        #avg_data = weighted_sum(base_data_avg, comp_data_avg, weight)
-#        weight = comp_weight(comp_data_avg, scale_factor = scale_factor, mode = mode)
-#        #weight = comp_weight2(base_data, comp_data , comp_data_avg)  
-#
-#    if multitau == True:
-#        n = weight.shape[-1]
-#        # make sure weights are increasing.
-#        for i,w in enumerate(weight):
-#            if i > 0:
-#                #make first weight equal last weight from previous level
-#                weight[i,...,n//2] = weight[i-1,...,-1]
-#                #this way it can only increase when going through level.
-#            increasing(w,out = w)   
-#    return weight
-
-
-def _data_estimator(data, size = 8, n = 3, multilevel = False):
-    from cddm.multitau import log_average, merge_multilevel
-    from cddm.avg import denoise
-    if multilevel == False:
-        x,y = log_average(data,size)
-    else:
-        x,y = merge_multilevel(data)
-    return x, denoise(y, n = n, out = y)
-
-
-#def average(x, size = 2):
-#    """Performs averaging of normalized linear-spaced data with delay-dependent
-#    kernel. 
-#    
-#    You must first normalize with :func:`.core.normalize` before averaging!
-#    
-#    Parameters
-#    ----------
-#    data : array
-#        Input array of linear-spaced data
-#    size : int
-#        Sampling size. Number of data points per each doubling of time.
-#        Any positive number is valid.
-#        
-#    Returns
-#    -------
-#    avg : ndarray
-#       Averaged data of same shape as the original data.
-#    """
-#        
-#    from cddm.multitau import log_average
-#    x = np.asarray(x)
-#    t,xl= log_average(x,size)
-#    new_t = np.arange(x.shape[-1])
-#    out = log_interpolate(new_t,t,xl)
-#    return out
+        return data    
 
 
 
-#if __name__ == "__main__":
-#    import doctest
-#    doctest.testmod()
-#   
\ No newline at end of file
