@@ -17,30 +17,38 @@ from __future__ import absolute_import, print_function, division
 import numpy as np
 from cddm.conf import FDTYPE
 
-from cddm._sim_nb import mirror, numba_seed, make_step, draw_points, draw_psf
+from cddm._sim_nb import mirror, numba_seed, make_step, draw_points, draw_psf, shot_noise_poisson, shot_noise_gaussian, adc_clean
+import cddm._sim_nb as _sim_nb
 
-def _inspect_frame_dtype(dtype):
+def _inspect_frame_dtype(dtype, intensity):
     dtype = np.dtype(dtype)
     if dtype not in ("uint8", "uint16"):
         raise ValueError("Unsupported dtype")
+    if intensity > 255 and dtype != "uint16" or intensity < 0:
+        raise ValueError("Invalid intensity for dtype {}".format(dtype))
     return dtype
 
-def form_factor(shape = (512,512), sigma = 3, intensity = 5, navg = 100, dtype = "uint8"):
+def form_factor(window, sigma = 3, intensity = 5, navg = 10, dtype = "uint8", mode = "rfft2"):
     """Computes point spread function form factor.
     
     Draws a PSF randomly in the frame and computes FFT, returns average absolute
-    of the FFT
+    of the FFT.
     
     Parameters
     ----------
-    shape : (int,int)
-        Frame  shape
+    window : ndarray
+        A 2D window function used in the analysis. Set this to np.ones if you do
+        not use one. 
     sigma : float
-        Sigma of the PSF
-    intensity : uint8
+        Sigma of the PSF of the image
+    intensity : unsigned int
         Intensity value
     navg : int
-        Specifies how many FFTs are averaged.
+        Specifies mesh size for averaging.
+    dtype : np.dtype
+        One of "uint8" or "uint16". Defines output dtype of the image.
+    mode : str, optional
+        Either 'rfft2' (default) or 'fft2'. 
         
     Returns
     -------
@@ -48,13 +56,24 @@ def form_factor(shape = (512,512), sigma = 3, intensity = 5, navg = 100, dtype =
         Average absolute value of the FFT of the PSF.
     """
     out = 0.
-    dtype = _inspect_frame_dtype(dtype)
+    window = np.asarray(window)
+    shape = window.shape
+    dtype = _inspect_frame_dtype(dtype, intensity)
+    
     for i in range(navg):
-        im = np.zeros(shape, dtype)
-        coord = np.random.rand(1,2) if navg > 1 else np.array([[0,0]],FDTYPE)
-        im = draw_psf(im, coord,np.array([intensity], dtype),np.array([sigma], FDTYPE))
-        out += np.abs(np.fft.rfft2(im))
-    return out/navg
+        for j in range(navg):
+            im = np.zeros(shape, dtype)
+            coord = (np.array(((i,j),)) * np.array(shape))*1.0 / navg
+            im = draw_psf(im, coord,np.array([intensity], dtype),np.array([sigma], FDTYPE))
+            im = im*window
+            if mode == 'rfft2':
+                out += np.abs(np.fft.rfft2(im))**2
+            elif mode == "fft2":
+                out += np.abs(np.fft.fft2(im))**2
+            else:
+                raise ValueError("Unknown mode.")
+             
+    return (out/navg**2)**0.5
 
 def seed(value):
     """Seed for numba and numpy random generator"""
@@ -111,7 +130,7 @@ def brownian_walk(x0, count = 1024, shape = (256,256), delta = 1, dt = 1, veloci
         x = make_step(x,scale, velocity) 
         x = mirror(x,x1,x2)
         
-def brownian_particles(count = 500, shape = (256,256),particles = 100, delta = 1, dt = 1,velocity = 0., x0 = None):
+def brownian_particles(count = 500, shape = (256,256),num_particles = 100, delta = 1, dt = 1,velocity = 0., x0 = None):
     """Coordinates generator of multiple brownian particles.
     
     Builds particles randomly distributed in the computation box and performs
@@ -123,7 +142,7 @@ def brownian_particles(count = 500, shape = (256,256),particles = 100, delta = 1
         Number of steps to calculate
     shape : (int,int)
         Shape of the box
-    particles : int
+    num_particles : int
         Number of particles in the box
     delta : float
         Step variance in pixel units (when dt = 1)
@@ -141,10 +160,10 @@ def brownian_particles(count = 500, shape = (256,256),particles = 100, delta = 1
         Length of the array equals number of particles.
     """
     if x0 is None:
-        x0 = np.asarray(np.random.rand(int(particles),2)*np.array(shape),FDTYPE)
+        x0 = np.asarray(np.random.rand(int(num_particles),2)*np.array(shape),FDTYPE)
     else:
         x0 = np.asarray(x0,FDTYPE)
-        if len(x0) != particles:
+        if len(x0) != num_particles:
             raise ValueError("Wrong length of initial coordinates x0")
  
     v0 = np.zeros_like(x0)
@@ -153,7 +172,7 @@ def brownian_particles(count = 500, shape = (256,256),particles = 100, delta = 1
         yield data
              
 def particles_video(particles, t1, shape = (256,256), t2 = None, 
-                 background = 0, intensity = 10, sigma = None, noise = 0., dtype = "uint8"):
+                 background = 0, intensity = 10, sigma = None, dtype = "uint8"):
     """Creates brownian particles video
     
     Parameters
@@ -172,8 +191,6 @@ def particles_video(particles, t1, shape = (256,256), t2 = None,
         Peak Intensity of the particle.
     sigma : float
         Sigma of the gaussian spread function for the particle
-    noise : float, optional
-        Intensity of the random noise
     dtype : dtype
         Numpy dtype of frames, either uint8 or uint16
         
@@ -182,7 +199,7 @@ def particles_video(particles, t1, shape = (256,256), t2 = None,
     frames : tuple of ndarrays
         A single-frame or dual-frame images (ndarrays).
     """
-    dtype = _inspect_frame_dtype(dtype)
+    dtype = _inspect_frame_dtype(dtype,intensity)
         
     background = np.zeros(shape = shape,dtype = dtype) + background
     height, width = shape
@@ -194,11 +211,8 @@ def particles_video(particles, t1, shape = (256,256), t2 = None,
     t1 = list(t1)
     t2 = list(t2) if t2 is not None else None
         
-    def get_frame(data, noise):
+    def get_frame(data):
         im = background.copy()
-        if noise > 0.:
-            n = np.asarray(np.random.poisson(noise, shape),dtype = dtype)
-            im = np.add(im, n, im)
 
         if sigma is None:
             _int = np.asarray(intensity, dtype)
@@ -221,11 +235,11 @@ def particles_video(particles, t1, shape = (256,256), t2 = None,
     
         if i in t1[:1]:
             t1.pop(0)
-            out1 = get_frame(data,noise)   
+            out1 = get_frame(data)   
             
         if t2 is not None and i in t2[:1]:
             t2.pop(0)
-            out2 = get_frame(data,noise)  
+            out2 = get_frame(data)  
             
         if t2 is not None:
             if out1 is not None and out2 is not None:
@@ -262,12 +276,12 @@ def data_trigger(data, indices):
                 #all done.
                 break
             
-def plot_random_walk(count = 5000, particles = 2, shape = (256,256)):
+def plot_random_walk(count = 5000, num_particles = 2, shape = (256,256)):
     """Brownian particles usage example. Track 2 particles"""
     import matplotlib.pyplot as plt 
-    x = np.array([x for x in brownian_particles(count = count, particles = particles, shape = shape)])
+    x = np.array([x for x in brownian_particles(count = count, num_particles = num_particles, shape = shape)])
         
-    for i in range(particles): 
+    for i in range(num_particles): 
         
         # Plot the 2D trajectory.
         plt.plot(x[:,i,0],x[:,i,1])
@@ -287,8 +301,24 @@ def plot_random_walk(count = 5000, particles = 2, shape = (256,256)):
     plt.ylim(0,shape[0])
     plt.axis('equal')
     plt.grid(True)
+    
+def create_random_time(nframes, n = 32, dt_min = 1):
+    """Create trigger time for single-camera random ddm experiments"""
+    t0 = np.arange(nframes)*n
+    r1 = 0
+    for i in range(nframes):
+        r1 = np.random.randint(max(0,dt_min-n+r1), n)
+        t0[i] += r1
+    return t0  
  
-def create_random_times1(nframes,n = 20):
+def random_time_count(nframes, n = 32):
+    """Returns estimated count for single-camera random triggering scheme"""
+    count = (np.arange(nframes*n+1)[::-1])/n/n
+    count[0:n] = np.arange(n*1.)/(n) * count[0:n]
+    count[0] = nframes * n
+    return count
+ 
+def create_random_times1(nframes,n = 32):
     """Create trigger times for c-ddm experiments based on Eq.7 from the paper"""
     iperiod = n * 2
     t0 = np.arange(nframes)*iperiod
@@ -298,7 +328,7 @@ def create_random_times1(nframes,n = 20):
     t2 = t0 + r2
     return t1, t2  
 
-def create_random_times2(nframes,n = 20):
+def create_random_times2(nframes,n = 32):
     """Create trigger times for c-ddm experiments based on Eq.8 from the paper"""
     iperiod = n * 2
     t0 = np.arange(nframes)*iperiod
@@ -310,10 +340,9 @@ def create_random_times2(nframes,n = 20):
     t2[mask] = t1[mask]
     return t1, t2 
 
-def simple_brownian_video(t1, t2 = None, shape = (256,256), background = 0, intensity = 5, sigma = 3, noise = 0, dtype = "uint8", **kw):
+def simple_brownian_video(t1, t2 = None, shape = (256,256), background = 0, intensity = 5, sigma = 3, dtype = "uint8", **kw):
     """DDM or c-DDM video generator.
-
-    
+ 
     Parameters
     ----------
     t1 : array-like
@@ -328,8 +357,6 @@ def simple_brownian_video(t1, t2 = None, shape = (256,256), background = 0, inte
         Peak Intensity of the particle.
     sigma : float
         Sigma of the gaussian spread function for the particle
-    noise : float, optional
-        Intensity of the random noise
     dtype : dtype
         Numpy dtype of frames, either uint8 or uint16
     kw : extra arguments
@@ -350,8 +377,58 @@ def simple_brownian_video(t1, t2 = None, shape = (256,256), background = 0, inte
     kw["count"] = count 
     kw["shape"] = shape
     p = brownian_particles(**kw) 
-    return particles_video(p, t1 = t1, t2 = t2, shape = shape, sigma = sigma, background = background,intensity = intensity, noise = noise, dtype = dtype) 
+    return particles_video(p, t1 = t1, t2 = t2, shape = shape, sigma = sigma, background = background,intensity = intensity, dtype = dtype) 
 
+    
+def adc(frame,  saturation = 32768, black_level = 0, bit_depth = "14bit", 
+        readout_noise = 0., noise_model = "gaussian", out = None):
+    """Simulated ADC conversion process of ideal signal.
+    
+    It applies shot noise with the standard deviation of the square of the
+    provided signal and adds a readout_noise of the provided mean value and 
+    shot noise characteristics.
+    
+    Parameters
+    ----------
+    frame : ndarray
+        Input noisless signal
+    saturation : int
+        Defines the saturation value of the sensor
+    black_level : int
+        Defines black level subtraction value. 
+    bit_depth : str
+        ADC bit depth. Either '8bit', '10bit', '12bit' or '14bit'. This defines
+        what kind of scalling is performed when converting results to uint16
+        (or uint8) image. 
+    readout_noise : float
+        Value of the additional noise added to the frame.
+    noise_model : str
+        Either 'gaussian' or 'poisson' or 'none' to disable noise
+    out : ndarray, optional
+        Output array.
+        
+    Returns
+    -------
+    frame : ndarray
+        Noissy image
+    """
+    readout_noise = np.array(readout_noise, FDTYPE)
+    if noise_model == "gaussian":
+        frame = shot_noise_gaussian(frame, readout_noise)
+    elif noise_model == "poisson":
+        frame = shot_noise_poisson(frame, readout_noise)
+    elif noise_model != "none":
+        raise ValueError("Invalid noise_model")
+    max_value = 2**int(bit_depth[:-3]) - 1
+    if max_value < 256:
+        max_value = np.uint8(max_value)
+    else:
+        max_value = np.uint16(max_value)
+    return adc_clean(frame,saturation,black_level, max_value, out = out)
+        
+
+
+        
 #
 #if __name__ == "__main__":
 #    import doctest
