@@ -22,6 +22,8 @@ except ImportError:
     #python 2.7
     from Queue import Queue
 
+import threading
+
 if CV2_INSTALLED:
     import cv2
     
@@ -326,7 +328,7 @@ def multiply(x,y, inplace = False, dtype = None):
             yield tuple((np.asarray(frame*w, dtype = dtype) for w, frame in zip(arrays,frames)))
             
 class ImageShow():
-    """A simple interface for video visualization using matplotlib opencv or
+    """A simple interface for video visualization using matplotlib, opencv, or
     pyqtgraph.
     
     Parameters
@@ -393,12 +395,13 @@ class ImageShow():
             self._mpl_imshow(im)
             
     def __del__(self):
-        if CDDMConfig.showlib == "cv2":
-            cv2.destroyWindow(self.fig)
-        elif CDDMConfig.showlib == "pyqtgraph":
-            self.fig.destroy()
-        else:
-            plt.close(self.fig)       
+        if self.fig is not None:
+            if CDDMConfig.showlib == "cv2":
+                cv2.destroyWindow(self.fig)
+            elif CDDMConfig.showlib == "pyqtgraph":
+                self.fig.destroy()
+            else:
+                plt.close(self.fig)       
     
 def pause(i = 1):
     """Pause in milliseconds needed to update matplotlib or opencv figures
@@ -409,6 +412,8 @@ def pause(i = 1):
         plt.pause(i/1000.)
     else: 
         app = QtGui.QApplication.instance()
+        if app is None:
+            app = QtGui.QApplication([])
         app.processEvents()
         
 #placehold for imshow figures        
@@ -419,7 +424,7 @@ def play(video, fps = 100., max_delay = 0.1):
     
     You must first call show functions (e.g. :func:`show_video`) to specify 
     what needs to be played. This function performs the actual display when in
-    a for loop
+    a for loop.
     
     Parameters
     ----------
@@ -450,6 +455,10 @@ def play(video, fps = 100., max_delay = 0.1):
     
     >>> v1,v2 = asarrays(play(video, fps = 30),count = 256)
     
+    See Also
+    --------
+    :func:`.video.play_threaded`
+    
     """
     t0 = None
     for i, frames in enumerate(video):   
@@ -468,6 +477,93 @@ def play(video, fps = 100., max_delay = 0.1):
         
     _FIGURES.clear()
     
+
+def play_threaded(video, fps = None):
+    """Plays video for real-time visualization. 
+    
+    You must first call show functions (e.g. :func:`show_video`) to specify 
+    what needs to be played. This function performs the actual display when in
+    a for loop. It works similar to :func:`play`, but it first creates a thread
+    and starts the video iterator in the background. 
+    
+    Parameters
+    ----------
+    video : iterable
+        A multi-frame iterable object. 
+    fps : float, optional
+        Desired video fps for display. This may be different from the actual 
+        video fps. If not set, it will display video as fast as possible.
+        
+    Returns
+    -------
+    video : iterator
+        A multi-frame iterator    
+        
+    Examples
+    --------
+    
+    First create some test data of a dual video
+    
+    >>> video = random_video(count = 256, dual = True)
+    >>> video = show_video(video)
+    
+    Now we can load video to memory, and play it as we load frame by frame...
+    
+    >>> v1,v2 = asarrays(play_threaded(video, fps = 30),count = 256)
+
+    See Also
+    --------
+    :func:`.video.play`
+    
+    """
+    q = Queue()
+    
+    def worker(video):
+        try:
+            for frames in video: 
+                q.put(frames)
+        finally:
+            q.put(None)
+    
+    threading.Thread(target=worker,args = (video,) ,daemon=True).start()
+    out = False #dummy  
+    t0 = None   
+    i = 0
+    while True:
+        if fps is None:
+            while not q.empty():
+                out = q.get()
+                q.task_done()
+                if out is None:
+                    break
+                else:
+                    yield out
+        else:
+            while True:
+                out = q.get()
+                q.task_done()
+                if t0 is None:
+                    t0 = time.time()
+                if out is None:
+                    break
+                else:
+                    yield out
+                if time.time()-t0 >= i/fps:
+                    break            
+                
+        if out is None:
+            break
+
+        for key in list(_FIGURES.keys()):
+            (viewer, queue) = _FIGURES.get(key)
+            if not queue.empty():
+                viewer.show(queue.get())
+                queue.task_done()
+        i+=1
+        pause()
+            
+    _FIGURES.clear()
+        
 def figure_title(name):
     """Generate a unique figure title"""
     i = len(_FIGURES)+1
@@ -566,7 +662,7 @@ def show_video(video, id = 0, title = None, norm_func = lambda x : x.real):
             queue.put(frames[id],block = False)
         yield frames  
      
-def show_diff(video, title = None):
+def show_diff(video, title = None, normalize = False, dt = None, t1 = None, t2 = None):
     """Returns a video and performs image difference live video show.
     This works in connection with :func:`play` that does the actual display.
     
@@ -577,12 +673,26 @@ def show_diff(video, title = None):
     title : str
         Unique title of the video. You can use :func:`figure_title`
         a to produce unique name.
+    normalize : bool
+        Whether to normalize frames to its mean value before subtracting. Note
+        that this does not normalize the output video, only the displayed video
+        is normalized.
         
     Returns
     -------
     video : iterator
         A multi-frame iterator
     """
+    def _process_frames(frames, queue):
+        if queue.empty():
+            x,y = frames
+            x,y = x.real, y.real
+            if normalize == True:
+                x = x/x.mean()
+                y = y/y.mean()
+            m = 2* max(x.max(),y.max())
+            im = x/m - y/m + 0.5
+            queue.put(im,block = False)        
     
     if title is None:
         title = figure_title("difference video")
@@ -591,33 +701,43 @@ def show_diff(video, title = None):
     queue = Queue(1)
     _FIGURES[title] = (viewer, queue)
     
-    for frames in video:
-        if queue.empty():
-            x,y = frames
-            x,y = x.real, y.real
-            m = 2* max(x.max(),y.max())
-            im = x/m - y/m + 0.5
-            queue.put(im,block = False)
-        yield frames    
+    if dt is None:
+        for frames in video:
+            _process_frames(frames, queue)
+            yield frames 
+    else:
+        for frames,_t1,_t2 in zip(video,t1,t2):
+            if abs(_t2-_t1) in dt:
+                _process_frames(frames, queue)
+            yield frames         
+        
 
 def random_video(shape = (512,512), count = 256, dtype = FDTYPE, max_value = 1., dual = False):
     """Random multi-frame video generator, useful for testing."""
     nframes = 2 if dual == True else 1 
     for i in range(count):
+        time.sleep(0.02)
         yield tuple((np.asarray(np.random.rand(*shape)*max_value,dtype) for i in range(nframes)))
 
         
-#if __name__ == '__main__':
-#    import cddm.conf
-#    cddm.conf.set_verbose(2)
-#
-#    #cddm.conf.set_showlib("pyqtgraph")    
-#    #example how to use show_video and play
-#    video = random_video(count = 256, dual = True)
-#    video = show_video(video)
-#    video = show_diff(video)
-#  
-#    v1,v2 = asarrays(play(video, fps = 30),count = 256)
+if __name__ == '__main__':
+    
+    
+    import cddm.conf
+    cddm.conf.set_verbose(2)
+
+
+    cddm.conf.set_showlib("pyqtgraph")    
+    #example how to use show_video and play
+    video = random_video(count = 1256, dual = True)
+    #video = load(video, 1256)
+    video = show_video(video)
+    video = show_diff(video)
+
+    #p = play_threaded(video)
+    #v1,v2 = asarrays(video,count = 1256)
+    #v1,v2 = asarrays(play(video,fps = 50),count = 1256)
+    v1,v2 = asarrays(play_threaded(video),count = 1256)
 ##    #example how to use ImageShow
 ##    video = random_video(count = 256)
 ##    viewer = ImageShow()
