@@ -6,7 +6,7 @@ import matplotlib.pyplot as plt
 from matplotlib.widgets import Slider, Button, RadioButtons, CheckButtons
 from cddm.map import rfft2_kangle,  sector_indexmap
 from cddm.multitau import normalize_multi, log_merge, log_average
-from cddm.core import normalize, _default_norm_from_data, _method_from_data, NORM_COMPENSATED, NORM_SUBTRACTED, NORM_WEIGHTED
+from cddm.norm import normalize, _default_norm_from_data, _method_from_data, NORM_STRUCTURED, NORM_SUBTRACTED, NORM_WEIGHTED, NORM_STANDARD, NORM_COMPENSATED
 from cddm.print_tools import disable_prints, enable_prints
 from cddm.decorators import doc_inherit, skip_runtime_error
 
@@ -214,24 +214,27 @@ class DataViewer(object):
     def _init_map(self):
         self.kmap, self.anglemap = rfft2_kangle(self.kisize, self.kjsize, self.shape)
         self.indexmap = sector_indexmap(self.kmap, self.anglemap, self.angle, self.sector, self.kstep)
-     
+
+    def _init_graph(self):   
+        self.graph = np.zeros(self.indexmap.shape)
+        
+        max_graph_value =self.kmap.max() #max(self.kmap[0,:].max(), self.kmap[:,0].max()) 
+        self.graph[0,0] = max_graph_value 
+        
+        self._max_graph_value = max_graph_value        
+
     def _init_fig(self):
         if self.data is None:
             raise ValueError("You must first set data to plot!")
         self._init_map()
+        self._init_graph()
         
         self.fig, (self.ax1,self.ax2) = plt.subplots(1,2,gridspec_kw = {'width_ratios':[3, 1]})
         
         self.fig.show()
         
         plt.subplots_adjust(bottom=0.25)
-        
-        self.graph = np.zeros(self.indexmap.shape)
-        
-        max_graph_value =self.kmap.max() #max(self.kmap[0,:].max(), self.kmap[:,0].max()) 
-        self.graph[0,0] = max_graph_value 
-        
-        self._max_graph_value = max_graph_value
+
         
         self.im = self.ax2.imshow(np.fft.fftshift(self.graph,0), 
             extent=[0,self.graph.shape[1],self.graph.shape[0]//2+1,-self.graph.shape[0]//2-1])
@@ -418,22 +421,24 @@ class CorrViewer(DataViewer):
         #self.rax = plt.axes([0.48, 0.55, 0.15, 0.3])
         self.cax = plt.axes([0.44, 0.72, 0.2, 0.15])
         
-        self.active = [bool(self.norm & 1),bool(self.norm & 2), bool(self.norm & 4)  ]
+        self.active = [bool(self.norm & NORM_STRUCTURED),bool(self.norm & NORM_SUBTRACTED), bool((self.norm & NORM_WEIGHTED == NORM_WEIGHTED)) , bool((self.norm & NORM_COMPENSATED) == NORM_COMPENSATED)  ]
         
-        self.check = CheckButtons(self.cax, ("compensated", "subtracted", "weighted"), self.active)
+        self.check = CheckButtons(self.cax, ("structured", "subtracted", "weighted", "compensated"), self.active)
         
         #self.radio = RadioButtons(self.rax,("norm 0","norm 1","norm 2","norm 3","norm 4", "norm 5", "norm 6", "norm 7"), active = self.norm, activecolor = "gray")
  
         def update(label):
-            index = ["compensated", "subtracted", "weighted"].index(label)
+            index = ["structured", "subtracted", "weighted", "compensated"].index(label)
             status = self.check.get_status()
-            norm = 0
-            if status[0]:
-                norm = norm | NORM_COMPENSATED
+
+            norm = NORM_STRUCTURED if status[0] == True else NORM_STANDARD    
+            
             if status[1]:
                 norm = norm | NORM_SUBTRACTED
             if status[2]:
                 norm = norm | NORM_WEIGHTED
+            if status[3]:
+                norm = norm | NORM_COMPENSATED
             try:
                 self.set_norm(norm)
             except ValueError:
@@ -482,7 +487,7 @@ class CorrViewer(DataViewer):
         if self.size is not None:
             t, data = log_average(data, self.size)
         else:
-            t = np.arange(len(data))
+            t = np.arange(data.shape[-1])
         
         return t, np.nanmean(data, axis = -2)
     
@@ -515,13 +520,27 @@ class MultitauViewer(CorrViewer):
         if mask is not None:
             self.kisize, self.kjsize = mask.shape
         
-    @doc_inherit
+    #@doc_inherit
     def set_norm(self, value):
+        """Sets norm parameter"""
         method = _method_from_data(self.data[0])
         self.norm = _default_norm_from_data(self.data[0],method,value)  
         
-    @doc_inherit
+    #@doc_inherit
     def set_data(self, data, background = None, variance = None):
+        """Sets correlation data.
+        
+        Parameters
+        ----------
+        data : tuple
+            A data tuple (as computed by ccorr, cdiff, adiff, acorr functions)
+        background : tuple or ndarray
+            Background data for normalization. For adiff, acorr functions this
+            is ndarray, for cdiff,ccorr, it is a tuple of ndarrays.
+        variance : tuple or ndarray
+            Variance data for normalization. For adiff, acorr functions this
+            is ndarray, for cdiff,ccorr, it is a tuple of ndarrays.
+        """
         self.data = data
         self.background = background
         self.variance = variance
